@@ -201,12 +201,10 @@ class Vport(object):
         check_addresses = []
         for port in self._api.snappi_config.ports:
             location = getattr(port, 'location', None)
-            if location is not None and re.search(
-                    "/|;", location) is not None:
-                if ';' in location:
-                    chassis_address = location.split(';')[0]
-                else:
-                    chassis_address = location.split('/')[0]
+            if location is not None:
+                location_info = self._api.parse_location_info(
+                        location)
+                chassis_address = location_info.chassis_info
                 chassis.find(Hostname='^%s$' % chassis_address)
                 if len(chassis) == 0:
                     add_addresses.append(chassis_address)
@@ -233,73 +231,6 @@ class Vport(object):
                             'After %s seconds, not all location hosts [%s] are reachable'
                             % (HostReadyTimeout, ', '.join(check_addresses)))
                     time.sleep(2)
-
-    def _get_layer1(self, port):
-        if len(self._api.snappi_config.layer1) == 0:
-            return
-        for layer1 in self._api.snappi_config.layer1:
-            for port_names in layer1.port_names:
-                if port.name in port_names:
-                    return layer1
-        
-    def _set_aggregation(self, port, imports):
-        """ If the card has multiple resource group to control speed within port
-        set it according to the speed"""
-        layer1 = self._get_layer1(port)
-        if layer1 is None:
-            return
-        location = port.location
-        if location is None:
-            return
-        if ';' not in location:
-            return
-        (hostname, cardid, portid) = location.split(';')
-        resource_group = None
-        speed_mode_map = Vport._SPEED_MODE_MAP
-        self._api.info("Checking port %s to set Layer1 speed %s" %
-                       (port.name, layer1.speed))
-        card_info = self._api.select_card_aggregation(location)
-        if 'aggregation' in card_info.keys() and len(card_info[
-                         'aggregation']) > 0:
-            for aggregation in card_info['aggregation']:
-                if portid.lstrip('0') in [res_port.split('/')[-1] for res_port in aggregation[
-                        'resourcePorts']]:
-                    self._reset_resource_mode(card_info, speed_mode_map)
-                    resource_group = aggregation
-                    break
-        
-        aggregation_mode = None
-        if resource_group is not None:
-            if layer1.speed in speed_mode_map:
-                mode = speed_mode_map[layer1.speed]
-                for available_mode in resource_group['availableModes']:
-                    if re.search(mode, available_mode.lower()) is not None:
-                        self._layer1_check.append(layer1.name)
-                        aggregation_mode = available_mode
-                        break
-            else:
-                self._api.warning("Speed %s not avialable within internal map" %
-                                  layer1.speed)
-        else:
-            self._api.warning("Please check physical port number for port %s" %
-                              port.name)
-
-        if aggregation_mode is not None:
-            if aggregation_mode != resource_group['mode']:
-                imports.append({
-                    'xpath': resource_group['xpath'],
-                    'mode': aggregation_mode
-                })
-    
-                self._api.info('Setting port %s to resource mode %s' %
-                               (port.name, aggregation_mode))
-            else:
-                self._api.info("Port %s already set to resource mode %s" %
-                               (port.name, aggregation_mode))
-        else:
-            self._api.warning("Speed %s not available within RG of port %s" %
-                              (layer1.speed, port.name))
-            
     
     def _set_location(self):
         location_supported = True
@@ -310,30 +241,10 @@ class Vport(object):
             location_supported = False
         
         self._add_hosts(60)
-        
-        # calling little bit costly operation? Otherwise we can't handle same port config for
-        # multiple run (_set_card_resource_mode reset to card level). Also some ports are
-        # handling multiple speed ('novusFourByTwentyFiveGigNonFanOut' and 'novusFourByTenGigNonFanOut')
-
-        # todo: restructure old module to getChassisWithDetailedResouceGroupsInfo
-        is_uhd = False
-        for port in self._api.snappi_config.ports:
-            location = port.location
-            if location is not None and '/' in location:
-                is_uhd = True
         with Timer(self._api,
                    'Aggregation mode speed change'):
-            if is_uhd is True:
-                layer1_check = self._api.resource_group.set_group()
-                self._layer1_check.extend(layer1_check)
-            else:
-                imports = []
-                for port in self._api.snappi_config.ports:
-                    self._set_aggregation(port, imports)
-                if self._import(imports) is False:
-                    self._api.info('Retrying card resource mode change')
-                    self._import(imports)
-            
+            layer1_check = self._api.resource_group.set_group()
+            self._layer1_check.extend(layer1_check)
         vports = self._api.select_vports()
         locations = []
         imports = []
@@ -452,13 +363,6 @@ class Vport(object):
         self._import(imports)
         
         self._set_auto_negotiation(vport, layer1, imports)
-
-    def _reset_resource_mode(self, card, speed_mode_map):
-        # add check for novus NOVUS10
-        novus10g_modes = ['normal', 'tenGigAggregation']
-        if re.search('novus', card['description'].lower()) and set(
-                novus10g_modes) == set(card['availableModes']):
-            speed_mode_map['speed_10_gbps'] = 'normal'
     
     def _set_card_resource_mode(self, vport, layer1, imports):
         """If the card has an aggregation mode set it according to the speed
